@@ -7,6 +7,14 @@ deactivate () {
         export PATH
         unset _OLD_VIRTUAL_PATH
     fi
+
+    # This should detect bash and zsh, which have a hash command that must
+    # be called to get it to forget past commands.  Without forgetting
+    # past commands the $PATH changes we made may not be respected
+    if [ -n "$BASH" -o -n "$ZSH_VERSION" ] ; then
+        hash -r
+    fi
+
     if [ -n "$_OLD_VIRTUAL_PS1" ] ; then
         PS1="$_OLD_VIRTUAL_PS1"
         export PS1
@@ -23,26 +31,26 @@ deactivate () {
 # unset irrelavent variables
 deactivate nondestructive
 
-export VIRTUAL_ENV="__VIRTUAL_ENV__"
+VIRTUAL_ENV="__VIRTUAL_ENV__"
+export VIRTUAL_ENV
 
 _OLD_VIRTUAL_PATH="$PATH"
-PATH="$VIRTUAL_ENV/bin:$PATH"
+PATH="$VIRTUAL_ENV/__BIN_NAME__:$PATH"
 export PATH
 
 _OLD_VIRTUAL_PS1="$PS1"
-if [ "`basename $VIRTUAL_ENV`" = "__" ] ; then
+if [ "`basename \"$VIRTUAL_ENV\"`" = "__" ] ; then
     # special case for Aspen magic directories
     # see http://www.zetadev.com/software/aspen/
-    PS1="[`basename \`dirname $VIRTUAL_ENV\``] $PS1"
+    PS1="[`basename \`dirname \"$VIRTUAL_ENV\"\``] $PS1"
 else
-    PS1="(`basename $VIRTUAL_ENV`)$PS1"
+    PS1="(`basename \"$VIRTUAL_ENV\"`)$PS1"
 fi
 export PS1
 
-# This should detect bash, which has a hash command that must
-# be called to get it to forget past commands.  Without
-# forgetting past commands the $PATH changes we made may not
-# be respected
-if [ -n "$BASH" ] ; then
+# This should detect bash and zsh, which have a hash command that must
+# be called to get it to forget past commands.  Without forgetting
+# past commands the $PATH changes we made may not be respected
+if [ -n "$BASH" -o -n "$ZSH_VERSION" ] ; then
     hash -r
 fi
